@@ -1,4 +1,4 @@
-import os, sys, math, urllib, re
+import os, math, urllib, re
 import toposort
 
 from dulwich.errors import HangupException
@@ -12,11 +12,6 @@ from mercurial.i18n import _
 from mercurial.node import hex, bin, nullid
 from mercurial import context, util as hgutil
 
-try:
-    from mercurial.error import RepoError
-except ImportError:
-    from mercurial.repo import RepoError
-
 
 class GitHandler(object):
 
@@ -33,19 +28,16 @@ class GitHandler(object):
 
         self.paths = ui.configitems('paths')
 
-        self.init_if_missing()
-        self.load_git()
         self.load_map()
         self.load_tags()
 
     # make the git data directory
     def init_if_missing(self):
-        if not os.path.exists(self.gitdir):
+        if os.path.exists(self.gitdir):
+            self.git = Repo(self.gitdir)
+        else:
             os.mkdir(self.gitdir)
-            Repo.init_bare(self.gitdir)
-
-    def load_git(self):
-        self.git = Repo(self.gitdir)
+            self.git = Repo.init_bare(self.gitdir)
 
     ## FILE LOAD AND SAVE METHODS
 
@@ -176,6 +168,8 @@ class GitHandler(object):
 
     def export_git_objects(self):
         self.ui.status(_("importing Hg objects into Git\n"))
+        self.init_if_missing()
+
         nodes = [self.repo.lookup(n) for n in self.repo]
         export = [node for node in nodes if not hex(node) in self._map_hg]
         total = len(export)
@@ -352,6 +346,8 @@ class GitHandler(object):
 
     def import_git_objects(self, remote_name=None, refs=None):
         self.ui.status(_("importing Git objects into Hg\n"))
+        self.init_if_missing()
+
         # import heads and fetched tags as remote references
         todo = []
         done = set()
@@ -543,7 +539,7 @@ class GitHandler(object):
             del new_refs['capabilities^{}']
             if not self.local_heads():
                 tip = hex(self.repo.lookup('tip'))
-                bookmarks.bookmark(self.ui, self.repo, 'master', tip)
+                bookmarks.bookmark(self.ui, self.repo, 'master', tip, force=True)
                 bookmarks.setcurrent(self.repo, 'master')
                 new_refs['refs/heads/master'] = self.map_git_get(tip)
 
@@ -598,9 +594,10 @@ class GitHandler(object):
             return want
         f, commit = self.git.object_store.add_pack()
         try:
-            return client.fetch_pack(path, determine_wants, graphwalker, f.write, self.ui.status)
-        except HangupException:
-            raise hgutil.Abort("the remote end hung up unexpectedly")
+            try:
+                return client.fetch_pack(path, determine_wants, graphwalker, f.write, self.ui.status)
+            except HangupException:
+                raise hgutil.Abort("the remote end hung up unexpectedly")
         finally:
             commit()
 
@@ -622,7 +619,10 @@ class GitHandler(object):
 
     def local_heads(self):
         try:
-            bms = bookmarks.parse(self.repo)
+            if getattr(bookmarks, 'parse', None):
+                bms = bookmarks.parse(self.repo)
+            else:
+                bms = self.repo._bookmarks
             return dict([(bm, hex(bms[bm])) for bm in bms])
         except AttributeError: #pragma: no cover
             return {}
@@ -659,7 +659,11 @@ class GitHandler(object):
 
     def update_hg_bookmarks(self, refs):
         try:
-            bms = bookmarks.parse(self.repo)
+            oldbm = getattr(bookmarks, 'parse', None)
+            if oldbm:
+                bms = bookmarks.parse(self.repo)
+            else:
+                bms = self.repo._bookmarks
             heads = dict([(ref[11:],refs[ref]) for ref in refs
                           if ref.startswith('refs/heads/')])
 
@@ -678,7 +682,11 @@ class GitHandler(object):
                         # fast forward
                         bms[head] = hgsha
             if heads:
-                bookmarks.write(self.repo, bms)
+                if oldbm:
+                    bookmarks.write(self.repo, bms)
+                else:
+                    self.repo._bookmarks = bms
+                    bookmarks.write(self.repo)
 
         except AttributeError:
             self.ui.warn(_('creating bookmarks failed, do you have'
@@ -838,7 +846,24 @@ class GitHandler(object):
         from dulwich.client import TCPGitClient, SSHGitClient, SubprocessGitClient
         for handler, transport in (("git://", TCPGitClient), ("git@", SSHGitClient), ("git+ssh://", SSHGitClient)):
             if uri.startswith(handler):
-                host, path = uri[len(handler):].split("/", 1)
-                return transport(host, thin_packs=False), '/' + path
+                # We need to split around : or /, whatever comes first
+                hostpath = uri[len(handler):]
+                if (hostpath.find(':') > 0 and hostpath.find('/') > 0):
+                    # we have both, whatever is first wins.
+                    if hostpath.find(':') < hostpath.find('/'):
+                      hostpath_seper = ':'
+                    else:
+                      hostpath_seper = '/'
+                elif hostpath.find(':') > 0:
+                    hostpath_seper = ':'
+                else:
+                    hostpath_seper = '/'
+
+                host, path = hostpath.split(hostpath_seper, 1)
+                if hostpath_seper == '/':
+                    transportpath = '/' + path
+                else:
+                    transportpath = path
+                return transport(host, thin_packs=False), transportpath
         # if its not git or git+ssh, try a local url..
         return SubprocessGitClient(thin_packs=False), uri
